/**
 * This handler process the inital installation event of the service worker
 * It allows the service worker become active as fast as possible
 */
self.addEventListener('install', (event) => {
  console.log('Installed dashboard SW');
  event.waitUntil(self.skipWaiting());
});

/**
 * This handler process the incoming push message and shows the push notification
 */
self.addEventListener('push', (event) => {
  if (event.data === null) {
    return;
  }

  console.log('[Service Worker] Push Received', event);

  const payload = event.data.json();
  const title = payload.title;

  event.waitUntil(
      self.registration.showNotification(title, payload)
  );
});

/**
 * This handler process the click event, if the user interacts with the push notification
 */
self.addEventListener('notificationclick', (event) => {
  console.log('[Service Worker] Notification click Received.', event);
  event.notification.close();

  const launchUrl = event.action || event.notification.data.launchUrl;

  if (launchUrl) {
    event.waitUntil(clients.openWindow(launchUrl));
  }
});
